@@ -1,6 +1,6 @@
 # This script generates the PyQt configuration and generates the Makefiles.
 #
-# Copyright (c) 2008 Riverbank Computing Limited <info@riverbankcomputing.com>
+# Copyright (c) 2009 Riverbank Computing Limited <info@riverbankcomputing.com>
 # 
 # This file is part of PyQt.
 # 
@@ -24,18 +24,12 @@
 # http://trolltech.com/products/qt/licenses/licensing/licensingoverview
 # or contact the sales department at sales@riverbankcomputing.com.
 # 
-# This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
-# INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
-# A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
-# granted herein.
-# 
 # This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 # WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 
 
 import sys
 import os
-import string
 import glob
 import optparse
 import shutil
@@ -44,13 +38,14 @@ import sipconfig
 
 
 # Initialise the globals.
-pyqt_version = 0x040402
-pyqt_version_str = "4.4.2"
+pyqt_version = 0x040600
+pyqt_version_str = "4.6"
 
-sip_min_version = 0x040705
+sip_min_version = 0x040900
 
 qt_version = 0
 qt_edition = ""
+qt_licensee = None
 qt_dir = None
 qt_incdir = None
 qt_libdir = None
@@ -78,7 +73,7 @@ dbuslibs = []
 # Under Windows qmake and the Qt DLLs must be into the system PATH otherwise
 # the dynamic linker won't be able to resolve the symbols.  On other systems we
 # assume we can just run qmake by using its full pathname.
-if sys.platform == "win32":
+if sys.platform == 'win32':
     MSG_CHECK_QMAKE = "Make sure you have a working Qt v4 qmake on your PATH."
 else:
     MSG_CHECK_QMAKE = "Make sure you have a working Qt v4 qmake on your PATH or use the -q argument to explicitly specify a working Qt v4 qmake."
@@ -92,7 +87,7 @@ def find_default_qmake():
     except KeyError:
         path = ""
 
-    if sys.platform == "win32":
+    if sys.platform == 'win32':
         base_qmake = "qmake.exe"
     else:
         base_qmake = "qmake"
@@ -168,11 +163,14 @@ def create_optparser():
             metavar="MODULE", dest="enabled", help="enable checks for the "
             "specified MODULE [default: checks for all modules will be "
             "enabled]")
+    g.add_option("--no-designer-plugin", action="store_false", default=True,
+            dest="designer_plugin", help="disable the building of the "
+            "Python plugin for Qt Designer [default: enabled]")
     g.add_option("-t", "--plugin", action="append", default=[],
             metavar="PLUGIN", dest="staticplugins", help="add PLUGIN to the "
             "list be linked (if Qt is built as static libraries)")
 
-    if sys.platform != "win32":
+    if sys.platform != 'win32':
         g.add_option("-q", "--qmake", action="callback", metavar="FILE",
                 default=qmake, dest="qmake", callback=store_abspath_file,
                 type="string",
@@ -184,6 +182,14 @@ def create_optparser():
             "[default: supplied by pkg-config]")
     p.add_option_group(g)
 
+    if sys.platform == 'darwin':
+        g = optparse.OptionGroup(p, title="MacOS X Configuration")
+        g.add_option("--use-arch", action="store", metavar="ARCH",
+                dest="use_arch", choices=["i386", "x86_64", "ppc"],
+                help="the architecture to use when running pyuic4 "
+                        "[default: system default]")
+        p.add_option_group(g)
+
     # Installation.
     g = optparse.OptionGroup(p, title="Installation")
     g.add_option("-b", "--bindir", action="callback",
@@ -198,7 +204,7 @@ def create_optparser():
             sipcfg.default_mod_dir)
     g.add_option("-p", "--plugin-destdir", action="callback", type="string",
             metavar="DIR", dest="plugindir", callback=store_abspath,
-            help="where the Designer plugin will be installed [default: "
+            help="where any plugins will be installed [default: "
             "QTDIR/plugins]")
     g.add_option("--no-sip-files", action="store_false", default=True,
             dest="install_sipfiles", help="disable the installation of the "
@@ -244,6 +250,41 @@ def create_optparser():
     return p
 
 
+class pyrccMakefile(sipconfig.ProgramMakefile):
+    """This class implements the Makefile for pyrcc.  This is specialised so
+    that pyrcc is automatically run against the examples.
+    """
+
+    def __init__(self):
+        sipconfig.ProgramMakefile.__init__(self, configuration=sipcfg,
+                build_file="pyrcc.sbf", dir="pyrcc",
+                install_dir=opts.pyqtbindir, console=1, qt=["QtCore", "QtXml"],
+                debug=opts.debug, warnings=1, universal=sipcfg.universal,
+                arch=sipcfg.arch)
+
+    def generate_target_default(self, mfile):
+        """Generate the default target."""
+        sipconfig.ProgramMakefile.generate_target_default(self, mfile)
+
+        # The correct call to pyrcc depends on the Python version.
+        if sys.hexversion >= 0x03000000:
+            flag = "-py3"
+        else:
+            flag = "-py2"
+
+        exe = "$(TARGET)"
+        if sys.platform != 'win32':
+            exe = "./" + exe
+
+        # Find all the .qrc files in the examples.
+        for root, _, files in os.walk("examples"):
+            rel_root = os.path.join("..", root)
+
+            for fn in files:
+                if fn.endswith(".qrc"):
+                    mfile.write("\t%s %s -o %s %s\n" % (exe, flag, os.path.join(rel_root, fn[:-4] + "_rc.py"), os.path.join(rel_root, fn)))
+
+
 class ConfigurePyQt4:
     """This class defines the methods to configure PyQt4.
     """
@@ -264,7 +305,9 @@ class ConfigurePyQt4:
             0x040300: "Qt_4_2_2",
             0x040303: "Qt_4_3_0",
             0x040400: "Qt_4_3_3",
-            0x050000: "Qt_4_4_0"
+            0x040401: "Qt_4_4_0",
+            0x040500: "Qt_4_4_1",
+            0x050000: "Qt_4_5_0"
         }
 
     def check_modules(self):
@@ -289,6 +332,8 @@ class ConfigurePyQt4:
         check_module("QtNetwork", "qhostaddress.h", "new QHostAddress()")
         check_module("QtOpenGL", "qgl.h", "new QGLWidget()")
         check_module("QtScript", "qscriptengine.h", "new QScriptEngine()")
+        check_module("QtScriptTools", "qscriptenginedebugger.h",
+                "new QScriptEngineDebugger()")
         check_module("QtSql", "qsqldatabase.h", "new QSqlDatabase()",
                 extra_libs=sql_libs)
         check_module("QtSvg", "qsvgwidget.h", "new QSvgWidget()")
@@ -323,6 +368,11 @@ class ConfigurePyQt4:
         sp_libs, sp_libdirs = self._static_plugins("QtCore")
         sp_incdirs = []
 
+        qpy_inc_dir, qpy_lib_dir = self._qpy_directories("QtCore")
+        sp_incdirs.append(qpy_inc_dir)
+        sp_libdirs.append(qpy_lib_dir)
+        sp_libs.append("qpycore")
+
         if opts.vendorcheck:
             sp_incdirs.append(opts.vendincdir)
             sp_libdirs.append(opts.vendlibdir)
@@ -340,15 +390,22 @@ class ConfigurePyQt4:
 
         if "QtGui" in pyqt_modules:
             sp_libs, sp_libdirs = self._static_plugins("QtGui")
+            sp_incdirs = []
+
+            qpy_inc_dir, qpy_lib_dir = self._qpy_directories("QtGui")
+            sp_incdirs.append(qpy_inc_dir)
+            sp_libdirs.append(qpy_lib_dir)
+            sp_libs.append("qpygui")
 
             if opts.bigqt:
+                cons_xtra_incdirs.extend(sp_incdirs)
                 cons_xtra_libdirs.extend(sp_libdirs)
                 cons_xtra_libs.extend(sp_libs)
 
                 generate_code("QtGui")
             else:
-                generate_code("QtGui", extra_lib_dirs=sp_libdirs,
-                        extra_libs=sp_libs)
+                generate_code("QtGui", extra_include_dirs=sp_incdirs,
+                        extra_lib_dirs=sp_libdirs, extra_libs=sp_libs)
 
         if "QtHelp" in pyqt_modules:
             generate_code("QtHelp")
@@ -362,6 +419,9 @@ class ConfigurePyQt4:
         if "QtScript" in pyqt_modules:
             generate_code("QtScript")
 
+        if "QtScriptTools" in pyqt_modules:
+            generate_code("QtScriptTools")
+
         if "QtSql" in pyqt_modules:
             sp_libs, sp_libdirs = self._static_plugins("QtSql")
 
@@ -396,24 +456,16 @@ class ConfigurePyQt4:
             generate_code("QtAssistant")
 
         if "QtDesigner" in pyqt_modules:
-            qpy_dir = os.path.abspath(os.path.join("qpy", "QtDesigner"))
-
-            if sys.platform == "win32":
-                if opts.debug:
-                    qpy_lib_dir = os.path.join(qpy_dir, "debug")
-                else:
-                    qpy_lib_dir = os.path.join(qpy_dir, "release")
-            else:
-                qpy_lib_dir = qpy_dir
+            qpy_inc_dir, qpy_lib_dir = self._qpy_directories("QtDesigner")
 
             if opts.bigqt:
-                cons_xtra_incdirs.append(qpy_dir)
+                cons_xtra_incdirs.append(qpy_inc_dir)
                 cons_xtra_libdirs.append(qpy_lib_dir)
                 cons_xtra_libs.append("qpydesigner")
 
                 generate_code("QtDesigner")
             else:
-                generate_code("QtDesigner", extra_include_dirs=[qpy_dir],
+                generate_code("QtDesigner", extra_include_dirs=[qpy_inc_dir],
                         extra_lib_dirs=[qpy_lib_dir],
                         extra_libs=["qpydesigner"])
 
@@ -531,6 +583,24 @@ class ConfigurePyQt4:
 
         f.close()
 
+    def _qpy_directories(self, mname):
+        """Return a tuple of the directories containing the header files and
+        library for the support library for the given module.
+
+        mname is the name of the module.
+        """
+        qpy_dir = os.path.abspath(os.path.join("qpy", mname))
+
+        if sys.platform == 'win32':
+            if opts.debug:
+                qpy_lib_dir = os.path.join(qpy_dir, "debug")
+            else:
+                qpy_lib_dir = os.path.join(qpy_dir, "release")
+        else:
+            qpy_lib_dir = qpy_dir
+
+        return qpy_dir, qpy_lib_dir
+
     def _static_plugins(self, mname):
         """Return a tuple of the libraries (in platform neutral format) and the
         directories they are contained in for all the requested static plugins
@@ -602,10 +672,16 @@ class ConfigurePyQt4:
     def module_installs(self):
         return ["__init__.py", "pyqtconfig.py"]
 
-    def qpylibs(self):
-        # See which QPy libraries to build.
+    def qpy_libs(self):
+        # See which QPy support libraries to build.
         qpylibs = {}
 
+        if "QtCore" in pyqt_modules:
+            qpylibs["QtCore"] = "qpycore.pro"
+
+        if "QtGui" in pyqt_modules:
+            qpylibs["QtGui"] = "qpygui.pro"
+
         if "QtDesigner" in pyqt_modules:
             qpylibs["QtDesigner"] = "qpydesigner.pro"
 
@@ -613,38 +689,58 @@ class ConfigurePyQt4:
         qmake_args = fix_qmake_args()
         cwd = os.getcwd()
 
-        for qpy, pro in qpylibs.iteritems():
-            sipconfig.inform("Creating QPy library for %s Makefile..." % qpy)
+        for qpy, pro in qpylibs.items():
+            sipconfig.inform("Creating QPy support library for %s Makefile..." % qpy)
 
             os.chdir(os.path.join("qpy", qpy))
 
+            wrapped_pro = "w_" + pro
+
+            f = open(wrapped_pro, 'w+')
+
+            if sipcfg.arch:
+                f.write(arch_config())
+
             if sipcfg.universal:
-                upro = "u_" + pro
+                f.write("QMAKE_MAC_SDK = %s\n" % sipcfg.universal)
 
-                f = open(upro, 'w+')
+            inc_path = [sipcfg.py_inc_dir]
+            if qpy == "QtCore":
+                if sipcfg.sip_inc_dir != sipcfg.py_inc_dir:
+                    inc_path.insert(0, sipcfg.sip_inc_dir)
 
-                f.write(
-"""# Setup the normal .pro file for universal binaries.
+                if opts.bigqt:
+                    qtcore_dir = "../../_qt"
+                else:
+                    qtcore_dir = "../../QtCore"
 
-CONFIG += ppc i386
-QMAKE_MAC_SDK = %s
+                inc_path.append(qtcore_dir)
+
+            if opts.debug:
+                pro_config = 'debug'
+            else:
+                pro_config = 'release'
+
+            f.write(
+"""# Tell the original .pro file about additional directories.
+INCLUDEPATH = %s
+
+CONFIG += %s
 
 include(%s)
-""" % (sipcfg.universal, pro))
+""" % (" ".join(inc_path), pro_config, pro))
 
-                f.close()
+            f.close()
 
-                pro = upro
-
-            run_command("%s %s %s" % (opts.qmake, qmake_args, pro))
+            run_command("%s %s %s" % (opts.qmake, qmake_args, wrapped_pro))
             os.chdir(cwd)
 
-        sipconfig.inform("Creating QPy libraries Makefile...")
+        sipconfig.inform("Creating QPy support libraries Makefile...")
 
         sipconfig.ParentMakefile(
             configuration=sipcfg,
             dir="qpy",
-            subdirs=qpylibs.keys()
+            subdirs=list(qpylibs.keys())
         ).generate()
 
         return ["qpy"]
@@ -662,7 +758,8 @@ include(%s)
                 install_dir=pydbusmoddir,
                 qt=["QtCore"],
                 debug=opts.debug,
-                universal=sipcfg.universal
+                universal=sipcfg.universal,
+                arch=sipcfg.arch
             )
 
             add_makefile_extras(makefile, dbusincdirs, dbuslibdirs, dbuslibs)
@@ -688,8 +785,15 @@ include(%s)
         # not on Windows (so that normal console use will work).
         sipconfig.inform("Creating pyuic4 wrapper...")
 
+        if sys.platform == 'darwin':
+            gui = True
+            use_arch = opts.use_arch
+        else:
+            gui = False
+            use_arch = ''
+
         uicdir=os.path.join(pyqt_modroot, "uic")
-        wrapper = sipconfig.create_wrapper(os.path.join(uicdir, "pyuic.py"), os.path.join("pyuic", "pyuic4"), (sys.platform == "darwin"))
+        wrapper = sipconfig.create_wrapper(os.path.join(uicdir, "pyuic.py"), os.path.join("pyuic", "pyuic4"), gui, use_arch)
 
         sipconfig.inform("Creating pyuic4 Makefile...")
 
@@ -716,7 +820,8 @@ include(%s)
                 qt=["QtCore", "QtGui", "QtXml"],
                 debug=opts.debug,
                 warnings=1,
-                universal=sipcfg.universal
+                universal=sipcfg.universal,
+                arch=sipcfg.arch
             )
 
             makefile.generate()
@@ -724,30 +829,17 @@ include(%s)
 
             sipconfig.inform("Creating pyrcc4 Makefile...")
 
-            makefile = sipconfig.ProgramMakefile(
-                configuration=sipcfg,
-                build_file="pyrcc.sbf",
-                dir="pyrcc",
-                install_dir=opts.pyqtbindir,
-                console=1,
-                qt=["QtCore", "QtXml"],
-                debug=opts.debug,
-                warnings=1,
-                universal=sipcfg.universal
-            )
-
+            makefile = pyrccMakefile()
             makefile.generate()
             tool.append("pyrcc")
         else:
             sipconfig.inform("pylupdate4 and pyrcc4 will not be built because the Qt XML module is missing.")
 
-        if "QtDesigner" in pyqt_modules:
-            enabled = True
-
+        if opts.designer_plugin and "QtDesigner" in pyqt_modules:
             py_major = sipcfg.py_version >> 16
             py_minor = (sipcfg.py_version >> 8) & 0x0ff
 
-            if sys.platform == "win32":
+            if sys.platform == 'win32':
                 lib_dir_flag = quote("-L%s" % sipcfg.py_lib_dir)
                 link = "%s -lpython%d%d" % (lib_dir_flag, py_major, py_minor)
                 pysh_lib = "python%d%d.dll" % (py_major, py_minor)
@@ -761,16 +853,16 @@ include(%s)
                     # version.
                     link = "-framework Python"
                 elif ("--enable-shared" in ducfg.get("CONFIG_ARGS", "") and
-                      glob.glob("%s/lib/libpython%d.%d*" % (ducfg["prefix"], py_major, py_minor))):
-                    lib_dir_flag = quote("-L%s/lib" % ducfg["prefix"])
+                      glob.glob("%s/lib/libpython%d.%d*" % (ducfg["exec_prefix"], py_major, py_minor))):
+                    lib_dir_flag = quote("-L%s/lib" % ducfg["exec_prefix"])
                     link = "%s -lpython%d.%d" % (lib_dir_flag, py_major, py_minor)
                 else:
                     sipconfig.inform("Qt Designer plugin disabled because Python library is static")
-                    enabled = False
+                    opts.designer_plugin = False
 
                 pysh_lib = ducfg["LDLIBRARY"]
 
-            if enabled:
+            if opts.designer_plugin:
                 sipconfig.inform("Creating Qt Designer plugin Makefile...")
 
                 # Run qmake to generate the Makefile.
@@ -784,14 +876,17 @@ include(%s)
                 fin.close()
 
                 prj = prj.replace("@PYINCDIR@", quote(sipcfg.py_inc_dir))
+                prj = prj.replace("@PYINCDIR@", " ".join((quote(sipcfg.py_conf_inc_dir), quote(sipcfg.py_inc_dir))))
                 prj = prj.replace("@PYLINK@", link)
                 prj = prj.replace("@PYSHLIB@", pysh_lib)
                 prj = prj.replace("@QTPLUGINDIR@", quote(opts.plugindir + "/designer"))
 
                 fout = open("python.pro", "w+")
 
+                if sipcfg.arch:
+                    fout.write(arch_config())
+
                 if sipcfg.universal:
-                    fout.write("CONFIG += ppc i386\n")
                     fout.write("QMAKE_MAC_SDK = %s\n" % sipcfg.universal)
 
                 fout.write(prj)
@@ -805,6 +900,19 @@ include(%s)
         return tool
 
 
+def arch_config():
+    """Return the qmake CONFIG line for a MacOS binary."""
+
+    qmake_archs = []
+    for a in sipcfg.arch.split():
+        if a in ('i386', 'x86_64'):
+            qmake_archs.append('x86')
+        elif a == 'ppc':
+            qmake_archs.append('ppc')
+
+    return 'CONFIG += %s\n' % ' '.join(qmake_archs)
+
+
 def quote(path):
     """Return a path with quotes added if it contains spaces."""
     if " " in path:
@@ -828,6 +936,9 @@ def inform_user():
 
     sipconfig.inform("Qt v%s %sis being used." % (sipconfig.version_to_string(qt_version), edstr))
 
+    if qt_licensee:
+        sipconfig.inform("Qt is licensed to %s." % qt_licensee)
+
     if sys.platform == "darwin" and qt_framework:
         sipconfig.inform("Qt is built as a framework.")
 
@@ -836,9 +947,11 @@ def inform_user():
     sipconfig.inform("The %s Qt libraries are in %s." % (lib_type, qt_libdir))
     sipconfig.inform("The Qt binaries are in %s." % qt_bindir)
     sipconfig.inform("The Qt mkspecs directory is in %s." % qt_datadir)
-    sipconfig.inform("These PyQt modules will be built: %s." % string.join(pyqt_modules))
+    sipconfig.inform("These PyQt modules will be built: %s." % ", ".join(pyqt_modules))
     sipconfig.inform("The PyQt Python package will be installed in %s." % opts.pyqtmoddir)
-    sipconfig.inform("The Designer plugin will be installed in %s." % os.path.join(opts.plugindir, "designer"))
+
+    if opts.designer_plugin:
+        sipconfig.inform("The Designer plugin will be installed in %s." % os.path.join(opts.plugindir, "designer"))
 
     if opts.api:
         sipconfig.inform("The QScintilla API file will be installed in %s." % os.path.join(opts.qscidir, "api", "python"))
@@ -901,7 +1014,7 @@ def run_command(cmd):
     lout = fout.readline()
     while lout:
         if opts.verbose:
-            sys.stdout.write(lout)
+            sys.stdout.write(str(lout))
 
         lout = fout.readline()
 
@@ -942,7 +1055,8 @@ def compile_qt_program(name, mname, extra_include_dirs=None, extra_lib_dirs=None
     if mname == "QtWebKit":
         qt.append("QtCore")
 
-    makefile = sipconfig.ProgramMakefile(sipcfg, console=1, qt=qt, warnings=0, opengl=opengl, debug=opts.debug)
+    makefile = sipconfig.ProgramMakefile(sipcfg, console=1, qt=qt, warnings=0,
+            opengl=opengl, debug=opts.debug, arch=sipcfg.arch)
 
     add_makefile_extras(makefile, extra_include_dirs, extra_lib_dirs, extra_libs)
 
@@ -955,7 +1069,7 @@ def compile_qt_program(name, mname, extra_include_dirs=None, extra_lib_dirs=None
     if not os.access(exe, os.X_OK):
         return None
 
-    if sys.platform != "win32":
+    if sys.platform != 'win32':
         exe = "./" + exe
 
     return exe
@@ -1039,7 +1153,8 @@ def check_dbus():
         sipconfig.inform("DBus v1 does not seem to be installed.")
         return
 
-    for f in iflags.split():
+    # Using str() means it will work with both Python v2 and v3.
+    for f in str(iflags).split():
         if f.startswith("-I"):
             dbusincdirs.append(f[2:])
         elif f.startswith("-L"):
@@ -1147,7 +1262,7 @@ def set_sip_flags(pyqt):
         qt_sip_flags.append("VendorID")
 
     # Handle the platform tag.
-    if sys.platform == "win32":
+    if sys.platform == 'win32':
         plattag = "WS_WIN"
     elif sys.platform == "darwin":
         if "__USE_WS_X11__" in sipcfg.build_macros()["DEFINES"]:
@@ -1177,6 +1292,10 @@ def set_sip_flags(pyqt):
         qt_sip_flags.append("-x")
         qt_sip_flags.append("Py_DateTime")
 
+    if sipcfg.py_version < 0x030000:
+        qt_sip_flags.append("-x")
+        qt_sip_flags.append("Py_v3")
+
     # There is an issue creating QObjects while the GIL is held causing
     # deadlocks in multi-threaded applications.  We don't fully understand this
     # yet so we make sure we avoid the problem by always releasing the GIL.
@@ -1200,12 +1319,13 @@ def needed_qt_libs(mname, qt_libs):
         "QtNetwork": ["QtCore"],
         "QtOpenGL": ["QtGui"],
         "QtScript": ["QtCore"],
+        "QtScriptTools": ["QtScript", "QtGui", "QtCore"],
         "QtSql": ["QtGui"],
         "QtSvg": ["QtGui"],
         "QtTest": ["QtGui"],
         "QtWebKit": ["QtNetwork", "QtGui"],
         "QtXml": ["QtCore"],
-        "QtXmlPatterns": ["QtCore"],
+        "QtXmlPatterns": ["QtNetwork", "QtCore"],
         "phonon": ["QtGui"],
         "QtAssistant": ["QtNetwork", "QtGui"],
         "QtDesigner": ["QtGui"],
@@ -1306,9 +1426,9 @@ def generate_code(mname, extra_include_dirs=None, extra_lib_dirs=None, extra_lib
     argv.append("sip")
 
     # SIP assumes POSIX style path separators.
-    argv.append(string.join(["sip", mname, mname + "mod.sip"], "/"))
+    argv.append("/".join(["sip", mname, mname + "mod.sip"]))
 
-    cmd = string.join(argv)
+    cmd = " ".join(argv)
 
     if opts.verbose:
         sys.stdout.write(cmd + "\n")
@@ -1345,7 +1465,8 @@ def generate_code(mname, extra_include_dirs=None, extra_lib_dirs=None, extra_lib
         warnings=1,
         static=opts.static,
         debug=opts.debug,
-        universal=sipcfg.universal
+        universal=sipcfg.universal,
+        arch=sipcfg.arch
     )
 
     add_makefile_extras(makefile, extra_include_dirs, extra_lib_dirs, extra_libs)
@@ -1373,36 +1494,42 @@ def check_license():
         lname = "GNU General Public License"
         lfile = None
 
-    sipconfig.inform("This is the %s version of PyQt %s (licensed under the %s) for Python %s on %s." % (ltype, pyqt_version_str, lname, string.split(sys.version)[0], sys.platform))
+    sipconfig.inform("This is the %s version of PyQt %s (licensed under the %s) for Python %s on %s." % (ltype, pyqt_version_str, lname, sys.version.split()[0], sys.platform))
 
     # Common checks.
-    if qt_edition and ltype != "internal":
-        if qt_edition != "free" and ltype == "GPL":
-            sipconfig.error("This version of PyQt and the %s edition of Qt have incompatible licenses." % qt_edition)
+    if qt_licensee and ltype == "GPL":
+        sipconfig.error("This version of PyQt and the commercial version of Qt have incompatible licenses.")
 
     # Confirm the license if not already done.
     if not opts.license_confirmed:
-        print
-
         if ltype == "GPL":
-            print "Type '2' to view the GPL v2 license."
-            print "Type '3' to view the GPL v3 license."
+            loptions = """
+Type '2' to view the GPL v2 license.
+Type '3' to view the GPL v3 license.
+"""
         else:
-            print "Type 'L' to view the license."
+            loptions = """
+Type 'L' to view the license.
+"""
 
-        print "Type 'yes' to accept the terms of the license."
-        print "Type 'no' to decline the terms of the license."
-        print
+        sys.stdout.write(loptions)
+        sys.stdout.write("""Type 'yes' to accept the terms of the license.
+Type 'no' to decline the terms of the license.
+
+""")
 
         while 1:
+            sys.stdout.write("Do you accept the terms of the license? ")
+            sys.stdout.flush()
+
             try:
-                resp = raw_input("Do you accept the terms of the license? ")
+                resp = sys.stdin.readline()
             except KeyboardInterrupt:
                 raise SystemExit
             except:
                 resp = ""
 
-            resp = string.lower(string.strip(resp))
+            resp = resp.strip().lower()
 
             if resp == "yes":
                 break
@@ -1434,7 +1561,7 @@ def get_build_macros(overrides):
     overrides is a list of macros overrides from the user.
     """
     # Get the name of the qmake configuration file to take the macros from.
-    if "QMAKESPEC" in os.environ.keys():
+    if "QMAKESPEC" in list(os.environ.keys()):
         fname = os.environ["QMAKESPEC"]
 
         if not os.path.dirname(fname):
@@ -1451,7 +1578,7 @@ def get_build_macros(overrides):
         sipconfig.error("Unable to find the qmake configuration file %s. Use the QMAKESPEC environment variable to specify the correct platform." % fname)
 
     # Add the Qt specific macros to the default.
-    names = sipcfg.build_macros().keys()
+    names = list(sipcfg.build_macros().keys())
     names.append("INCDIR_QT")
     names.append("LIBDIR_QT")
     names.append("MOC")
@@ -1503,7 +1630,7 @@ def fix_qmake_args(args=""):
     """
     if sys.platform == "darwin":
         # The Qt binary installer has macx-xcode as the default.
-        args = "-spec macx-g++ " + args
+        args = "-spec %s %s" % (sipcfg.platform, args)
 
     return args
 
@@ -1514,7 +1641,7 @@ def get_qt_configuration():
     """
     sipconfig.inform("Determining the layout of your Qt installation...")
 
-    if sys.platform == "win32":
+    if sys.platform == 'win32':
         # There is no -q flag so set the default explicitly.
         opts.qmake = find_default_qmake()
 
@@ -1530,7 +1657,7 @@ def get_qt_configuration():
     out_file = app + ".out"
     qmake_args = fix_qmake_args("-o " + make_file)
 
-    if sys.platform == "win32":
+    if sys.platform == 'win32':
         exe_file = os.path.join("release", app + ".exe")
         make_target = " release"
     elif sys.platform == "darwin":
@@ -1541,6 +1668,9 @@ def get_qt_configuration():
     # Generate the qmake project file.
     f = open(pro_file, "w")
 
+    if sipcfg.arch:
+        f.write(arch_config())
+
     f.write(
 """QT = core
 # This is for certain broken Linux distros and is needed to make sure that
@@ -1579,12 +1709,16 @@ int main(int, char **)
     out << QT_VERSION << '\\n';
     out << QT_EDITION << '\\n';
 
+    out << QLibraryInfo::licensee() << '\\n';
+
 #if defined(QT_SHARED) || defined(QT_DLL)
     out << "shared\\n";
 #else
     out << "\\n";
 #endif
 
+    // Determine which features should be disabled.
+
 #if defined(QT_NO_ACCESSIBILITY)
     out << "PyQt_Accessibility\\n";
 #endif
@@ -1609,10 +1743,17 @@ int main(int, char **)
     out << "PyQt_OpenSSL\\n";
 #endif
 
+#if defined(QT_NO_SIZEGRIP)
+    out << "PyQt_SizeGrip\\n";
+#endif
+
 #if !defined(QT3_SUPPORT) || QT_VERSION >= 0x040200
     out << "PyQt_NoPrintRangeBug\\n";
 #endif
 
+    if (sizeof (qreal) != sizeof (double))
+        out << "PyQt_qreal_double\\n";
+
     return 0;
 }
 """ % out_file)
@@ -1656,7 +1797,7 @@ int main(int, char **)
     f.close()
 
     global qt_dir, qt_incdir, qt_libdir, qt_bindir, qt_datadir, qt_pluginsdir
-    global qt_version, qt_edition, qt_shared, qt_xfeatures
+    global qt_version, qt_edition, qt_licensee, qt_shared, qt_xfeatures
 
     qt_dir = lines[0]
     qt_incdir = lines[1]
@@ -1666,8 +1807,12 @@ int main(int, char **)
     qt_pluginsdir = lines[5]
     qt_version = lines[6]
     qt_edition = lines[7]
-    qt_shared = lines[8]
-    qt_xfeatures = lines[9:]
+    qt_licensee = lines[8]
+    qt_shared = lines[9]
+    qt_xfeatures = lines[10:]
+
+    if qt_licensee == 'Open Source':
+        qt_licensee = None
 
     try:
         qt_version = int(qt_version)
@@ -1684,6 +1829,10 @@ int main(int, char **)
     if qt_edition & 0x200:
         # It has ActiveQt.
         qt_edition = "Desktop"
+
+        # ActiveQt became part of the open source version in v4.5.2.
+        if qt_version >= 0x040502 and qt_licensee is None:
+            qt_edition = "free"
     elif qt_edition & 0x008:
         # It has OpenGL.
         qt_edition = "free"
@@ -1794,7 +1943,7 @@ def main():
 
     sipconfig.ParentMakefile(
         configuration=sipcfg,
-        subdirs=pyqt.qpylibs() + pyqt_modules + xtra_modules + pyqt.tools(),
+        subdirs=pyqt.qpy_libs() + pyqt_modules + xtra_modules + pyqt.tools(),
         installs=installs
     ).generate()
 
@@ -1812,8 +1961,8 @@ if __name__ == "__main__":
     except SystemExit:
         raise
     except:
-        print \
+        sys.stderr.write(
 """An internal error occured.  Please report all the output from the program,
 including the following traceback, to support@riverbankcomputing.com.
-"""
+""")
         raise
