@@ -171,14 +171,22 @@ static int getToken()
                         return Tok_None;
                     break;
                 case 'Q':
-                    if ( strcmp(yyIdent + 1, "T_TR_NOOP") == 0 ) {
-                    yyParsingUtf8 = false;
+                    if (strcmp(yyIdent + 1, "T_TR_NOOP") == 0)
+                    {
+                        yyParsingUtf8 = false;
                         return Tok_tr;
-                    } else if ( strcmp(yyIdent + 1, "T_TRANSLATE_NOOP") == 0 ) {
-                    yyParsingUtf8 = false;
-                    return Tok_translate;
-                }
-                break;
+                    }
+                    else if (strcmp(yyIdent + 1, "T_TR_NOOP_UTF8") == 0)
+                    {
+                        yyParsingUtf8 = true;
+                        return Tok_trUtf8;
+                    }
+                    else if (strcmp(yyIdent + 1, "T_TRANSLATE_NOOP") == 0)
+                    {
+                        yyParsingUtf8 = false;
+                        return Tok_translate;
+                    }
+                    break;
                 case 'c':
                     if ( strcmp(yyIdent + 1, "lass") == 0 )
                         return Tok_class;
@@ -221,7 +229,6 @@ static int getToken()
         } else {
             switch ( yyCh ) {
                 case '#':
-                    yyCh = getChar();
                     do {
                         yyCh = getChar();
                     } while ( yyCh != EOF && yyCh != '\n' );
@@ -560,7 +567,7 @@ static void parse( MetaTranslator *tor, const char *initialContext,
                 break;
             case Tok_tr:
             case Tok_trUtf8:
-                utf8 = (yyTok == Tok_trUtf8);
+                utf8 = (yyTok == Tok_trUtf8 || (yyCodecForTr && strcmp(yyCodecForTr->name(), "UTF-8") == 0));
                 yyTok = getToken();
                 if (match(Tok_LeftParen) && matchString(&text))
                 {
@@ -600,7 +607,7 @@ static void parse( MetaTranslator *tor, const char *initialContext,
                     if (!text.isEmpty())
                     {
                         tor->insert(MetaTranslatorMessage(context, text, com, 
-                                yyFileName, yyLineNo,
+                                yyFileName, yyParenLineNo,
                                 QStringList(), utf8,
                                 MetaTranslatorMessage::Unfinished, plural));
                     }
@@ -648,7 +655,7 @@ static void parse( MetaTranslator *tor, const char *initialContext,
                     if (!text.isEmpty())
                     {
                         tor->insert( MetaTranslatorMessage(context, text, com, 
-                                                        yyFileName, yyLineNo,
+                                                        yyFileName, yyParenLineNo,
                                                         QStringList(), utf8,
                                                         MetaTranslatorMessage::Unfinished,
                                                         plural) );
@@ -675,7 +682,7 @@ static void parse( MetaTranslator *tor, const char *initialContext,
                         context = com.left( k );
                         com.remove( 0, k + 1 );
                         tor->insert( MetaTranslatorMessage(context, "", com,
-                                                        yyFileName, yyLineNo,
+                                                        yyFileName, yyParenLineNo,
                                                         QStringList(), false) );
     
                     }
