@@ -2,13 +2,12 @@ import sys
 import logging
 import os.path
 import re
-from itertools import count
 
 try:
-    from xml.etree.cElementTree import parse, SubElement
+    from xml.etree.ElementTree import parse, SubElement
 except ImportError:
     try:
-        from cElementTree import parse, SubElement
+        from ElementTree import parse, SubElement
     except ImportError:
         try:
             from elementtree.ElementTree import parse, SubElement
@@ -16,9 +15,9 @@ except ImportError:
             from PyQt4.elementtree.ElementTree import parse, SubElement
         
 
-from exceptions import NoSuchWidgetError
-from properties import Properties
-from objcreator import QObjectCreator
+from PyQt4.uic.exceptions import NoSuchWidgetError
+from PyQt4.uic.objcreator import QObjectCreator
+from PyQt4.uic.properties import Properties
 
 
 logger = logging.getLogger(__name__)
@@ -105,11 +104,16 @@ class UIParser(object):
         'foo1'
         """
         try:
-            return "%s%i" % (name, self.name_suffixes[name].next(),)
+            suffix = self.name_suffixes[name]
         except KeyError:
-            self.name_suffixes[name] = count(1)
+            self.name_suffixes[name] = 0
             return name
 
+        suffix += 1
+        self.name_suffixes[name] = suffix
+
+        return "%s%i" % (name, suffix)
+
     def reset(self):
         try: self.wprops.reset()
         except AttributeError: pass
@@ -144,101 +148,176 @@ class UIParser(object):
             
         self.column_counter = 0
         self.row_counter = 0
-        self.first_item = True
+        self.item_nr = 0
+        self.itemstack = []
+        self.sorting_enabled = None
         
         parent = self.stack.topwidget
         if isinstance(parent, (QtGui.QToolBox, QtGui.QTabWidget,
-                               QtGui.QStackedWidget)):
+                               QtGui.QStackedWidget, QtGui.QDockWidget,
+                               QtGui.QWizard)):
             parent = None
         
         
         self.stack.push(self.setupObject(widgetClass(elem), parent, elem))
 
         if isinstance(self.stack.topwidget, QtGui.QTableWidget):
-            self.stack.topwidget.clear()
             self.stack.topwidget.setColumnCount(len(elem.findall("column")))
             self.stack.topwidget.setRowCount(len(elem.findall("row")))
 
         self.traverseWidgetTree(elem)
         widget = self.stack.popWidget()
+
+        if isinstance(widget, QtGui.QTreeView):
+            self.handleHeaderView(elem, "header", widget.header())
+
+        elif isinstance(widget, QtGui.QTableView):
+            self.handleHeaderView(elem, "horizontalHeader",
+                    widget.horizontalHeader())
+            self.handleHeaderView(elem, "verticalHeader",
+                    widget.verticalHeader())
+
+        if self.sorting_enabled is not None:
+            widget.setSortingEnabled(self.sorting_enabled)
+            self.sorting_enabled = None
         
         if self.stack.topIsLayout():
-            self.stack.peek().addWidget(widget, *elem.attrib["grid-position"])
+            lay = self.stack.peek()
+            gp = elem.attrib["grid-position"]
 
-            
-        if isinstance(self.stack.topwidget, QtGui.QToolBox):
+            if isinstance(lay, QtGui.QFormLayout):
+                if gp[1]:
+                    role = QtGui.QFormLayout.FieldRole
+                else:
+                    role = QtGui.QFormLayout.LabelRole
+
+                lay.setWidget(gp[0], role, widget)
+            else:
+                lay.addWidget(widget, *gp)
+
+        topwidget = self.stack.topwidget
+
+        if isinstance(topwidget, QtGui.QToolBox):
             icon = self.wprops.getAttribute(elem, "icon")
             if icon is not None:
-                self.stack.topwidget.addItem(widget, icon, self.wprops.getAttribute(elem, "label"))
+                topwidget.addItem(widget, icon, self.wprops.getAttribute(elem, "label"))
             else:
-                self.stack.topwidget.addItem(widget, self.wprops.getAttribute(elem, "label"))
-                
-            tooltip = self.wprops.getAttribute(elem, "toolTip")
-            if tooltip is not None:
-                self.stack.topwidget.setItemToolTip(self.stack.topwidget.indexOf(widget), tooltip)
-                
-        elif isinstance(self.stack.topwidget, QtGui.QTabWidget):
-            icon = self.wprops.getAttribute(elem, "icon")
-            if icon is not None:
-                self.stack.topwidget.addTab(widget, icon, self.wprops.getAttribute(elem, "title"))
-            else:
-                self.stack.topwidget.addTab(widget, self.wprops.getAttribute(elem, "title"))
+                topwidget.addItem(widget, self.wprops.getAttribute(elem, "label"))
 
             tooltip = self.wprops.getAttribute(elem, "toolTip")
             if tooltip is not None:
-                self.stack.topwidget.setTabToolTip(self.stack.topwidget.indexOf(widget), tooltip)
+                topwidget.setItemToolTip(topwidget.indexOf(widget), tooltip)
+                
+        elif isinstance(topwidget, QtGui.QTabWidget):
+            icon = self.wprops.getAttribute(elem, "icon")
+            if icon is not None:
+                topwidget.addTab(widget, icon, self.wprops.getAttribute(elem, "title"))
+            else:
+                topwidget.addTab(widget, self.wprops.getAttribute(elem, "title"))
+
+            tooltip = self.wprops.getAttribute(elem, "toolTip")
+            if tooltip is not None:
+                topwidget.setTabToolTip(topwidget.indexOf(widget), tooltip)
             
-        elif isinstance(self.stack.topwidget, QtGui.QStackedWidget):
-            self.stack.topwidget.addWidget(widget)
+        elif isinstance(topwidget, QtGui.QWizard):
+            topwidget.addPage(widget)
             
-        elif isinstance(self.stack.topwidget, QtGui.QDockWidget):
-            self.stack.topwidget.setWidget(widget)
+        elif isinstance(topwidget, QtGui.QStackedWidget):
+            topwidget.addWidget(widget)
             
-        elif isinstance(self.stack.topwidget, QtGui.QMainWindow):
+        elif isinstance(topwidget, (QtGui.QDockWidget, QtGui.QScrollArea)):
+            topwidget.setWidget(widget)
+            
+        elif isinstance(topwidget, QtGui.QMainWindow):
             if type(widget) == QtGui.QWidget:
-                self.stack.topwidget.setCentralWidget(widget)
+                topwidget.setCentralWidget(widget)
             elif isinstance(widget, QtGui.QToolBar):
                 tbArea = self.wprops.getAttribute(elem, "toolBarArea")
 
                 if tbArea is None:
-                    self.stack.topwidget.addToolBar(widget)
+                    topwidget.addToolBar(widget)
                 else:
                     if isinstance(tbArea, basestring):
                         tbArea = getattr(QtCore.Qt, tbArea)
                     else:
                         tbArea = QtCore.Qt.ToolBarArea(tbArea)
 
-                    self.stack.topwidget.addToolBar(tbArea, widget)
+                    topwidget.addToolBar(tbArea, widget)
 
                 tbBreak = self.wprops.getAttribute(elem, "toolBarBreak")
 
                 if tbBreak:
-                    self.stack.topwidget.insertToolBarBreak(widget)
+                    topwidget.insertToolBarBreak(widget)
 
             elif isinstance(widget, QtGui.QMenuBar):
-                self.stack.topwidget.setMenuBar(widget)
+                topwidget.setMenuBar(widget)
             elif isinstance(widget, QtGui.QStatusBar):
-                self.stack.topwidget.setStatusBar(widget)
+                topwidget.setStatusBar(widget)
             elif isinstance(widget, QtGui.QDockWidget):
                 dwArea = self.wprops.getAttribute(elem, "dockWidgetArea")
-                self.stack.topwidget.addDockWidget(
-                    QtCore.Qt.DockWidgetArea(dwArea), widget)
+                topwidget.addDockWidget(QtCore.Qt.DockWidgetArea(dwArea),
+                        widget)
+
+    def handleHeaderView(self, elem, name, header):
+        value = self.wprops.getAttribute(elem, name + "Visible")
+        if value is not None:
+            header.setVisible(value)
+
+        value = self.wprops.getAttribute(elem, name + "CascadingSectionResizes")
+        if value is not None:
+            header.setCascadingSectionResizes(value)
+
+        value = self.wprops.getAttribute(elem, name + "DefaultSectionSize")
+        if value is not None:
+            header.setDefaultSectionSize(value)
+
+        value = self.wprops.getAttribute(elem, name + "HighlightSections")
+        if value is not None:
+            header.setHighlightSections(value)
+
+        value = self.wprops.getAttribute(elem, name + "MinimumSectionSize")
+        if value is not None:
+            header.setMinimumSectionSize(value)
+
+        value = self.wprops.getAttribute(elem, name + "ShowSortIndicator")
+        if value is not None:
+            header.setShowSortIndicator(value)
+
+        value = self.wprops.getAttribute(elem, name + "StretchLastSection")
+        if value is not None:
+            header.setStretchLastSection(value)
 
     def createSpacer(self, elem):
         width = int(elem.findtext("property/size/width"))
         height = int(elem.findtext("property/size/height"))
+
         sizeType = self.wprops.getProperty(elem, "sizeType")
         if sizeType is None:
             sizeType = "QSizePolicy::Expanding"
+
         policy = (QtGui.QSizePolicy.Minimum,
                   getattr(QtGui.QSizePolicy, sizeType.split("::")[-1]))
+
         if self.wprops.getProperty(elem, "orientation") == "Qt::Horizontal":
             policy = policy[1], policy[0]
-        spacer = self.factory.createQObject("QSpacerItem", self.uniqueName("spacerItem"),
-                                       (width, height) + policy,
-                                        is_attribute = False)
+
+        spacer = self.factory.createQObject("QSpacerItem",
+                self.uniqueName("spacerItem"), (width, height) + policy,
+                is_attribute=False)
+
         if self.stack.topIsLayout():
-            self.stack.peek().addItem(spacer, *elem.attrib["grid-position"])
+            lay = self.stack.peek()
+            gp = elem.attrib["grid-position"]
+
+            if isinstance(lay, QtGui.QFormLayout):
+                if gp[1]:
+                    role = QtGui.QFormLayout.FieldRole
+                else:
+                    role = QtGui.QFormLayout.LabelRole
+
+                lay.setItem(gp[0], role, spacer)
+            else:
+                lay.addItem(spacer, *gp)
 
     def createLayout(self, elem):
         # Qt v4.3 introduced setContentsMargins() and separate values for each
@@ -302,9 +381,44 @@ class UIParser(object):
             elem.attrib["name"] = classname[1:].lower()
         self.stack.push(self.setupObject(classname, parent, elem))
         self.traverseWidgetTree(elem)
+
         layout = self.stack.popLayout()
         if self.stack.topIsLayout():
-            self.stack.peek().addLayout(layout, *elem.attrib["grid-position"])
+            top_layout = self.stack.peek()
+            gp = elem.attrib["grid-position"]
+
+            if isinstance(top_layout, QtGui.QFormLayout):
+                if gp[1]:
+                    role = QtGui.QFormLayout.FieldRole
+                else:
+                    role = QtGui.QFormLayout.LabelRole
+
+                top_layout.setLayout(gp[0], role, layout)
+            else:
+                self.configureLayout(elem, layout)
+                top_layout.addLayout(layout, *gp)
+        else:
+            self.configureLayout(elem, layout)
+
+    def configureLayout(self, elem, layout):
+        if isinstance(layout, QtGui.QGridLayout):
+            self.setArray(elem, 'columnminimumwidth',
+                    layout.setColumnMinimumWidth)
+            self.setArray(elem, 'rowminimumheight',
+                    layout.setRowMinimumHeight)
+            self.setArray(elem, 'columnstretch', layout.setColumnStretch)
+            self.setArray(elem, 'rowstretch', layout.setRowStretch)
+
+        elif isinstance(layout, QtGui.QBoxLayout):
+            self.setArray(elem, 'stretch', layout.setStretch)
+
+    def setArray(self, elem, name, setter):
+        array = elem.attrib.get(name)
+        if array:
+            for idx, value in enumerate(array.split(',')):
+                value = int(value)
+                if value > 0:
+                    setter(idx, value)
 
     def handleItem(self, elem):
         if self.stack.topIsLayout():
@@ -312,70 +426,149 @@ class UIParser(object):
             self.traverseWidgetTree(elem)
         else:
             w = self.stack.topwidget
+
             if isinstance(w, QtGui.QComboBox):
+                text = self.wprops.getProperty(elem, "text")
                 icon = self.wprops.getProperty(elem, "icon")
+
                 if icon:
-                    w.addItem(icon, self.wprops.getProperty(elem, "text"))
+                    w.addItem(icon, '')
                 else:
-                    w.addItem(self.wprops.getProperty(elem, "text"))
+                    w.addItem('')
+
+                w.setItemText(self.item_nr, text)
+
             elif isinstance(w, QtGui.QListWidget):
-                if self.first_item:
-                    w.clear()
-                    self.first_item = False
-                item = self.factory.createQObject("QListWidgetItem",
-                                              self.uniqueName("item"),
-                                              (w,), False)
-                self.wprops.setProperties(item, elem)
+                text = self.wprops.getProperty(elem, "text")
+                icon = self.wprops.getProperty(elem, "icon")
+
+                if icon:
+                    item_name = "item"
+                else:
+                    item_name = None
+
+                item = self.factory.createQObject("QListWidgetItem", item_name,
+                        (w, ), False)
+
+                if self.item_nr == 0:
+                    self.sorting_enabled = self.factory.invoke("__sortingEnabled", w.isSortingEnabled)
+                    w.setSortingEnabled(False)
+
+                if text:
+                    w.item(self.item_nr).setText(text)
+
+                if icon:
+                    item.setIcon(icon)
+
             elif isinstance(w, QtGui.QTreeWidget):
-                if self.first_item:
-                    w.clear()
-                    self.first_item = False
-                    self.itemstack = [w]
+                if self.itemstack:
+                    parent, _ = self.itemstack[-1]
+                    _, nr_in_root = self.itemstack[0]
+                else:
+                    parent = w
+                    nr_in_root = self.item_nr
+
                 item = self.factory.createQObject("QTreeWidgetItem",
-                                              self.uniqueName("item"),
-                                              (self.itemstack[-1],), False)
+                        "item_%d" % len(self.itemstack), (parent, ), False)
+
+                if self.item_nr == 0 and not self.itemstack:
+                    self.sorting_enabled = self.factory.invoke("__sortingEnabled", w.isSortingEnabled)
+                    w.setSortingEnabled(False)
+
+                self.itemstack.append((item, self.item_nr))
+                self.item_nr = 0
+
+                # We have to access the item via the tree when setting the
+                # text.
+                titm = w.topLevelItem(nr_in_root)
+                for child, nr_in_parent in self.itemstack[1:]:
+                    titm = titm.child(nr_in_parent)
+
                 column = -1
                 for prop in elem.findall("property"):
+                    c_prop = self.wprops.convert(prop)
+
                     if prop.attrib["name"] == "text":
                         column += 1
-                        item.setText(column, self.wprops.convert(prop))
-                    else:
-                        item.setIcon(column, self.wprops.convert(prop))
-                self.itemstack.append(item)
+                        if c_prop:
+                            titm.setText(column, c_prop)
+                    elif c_prop:
+                        item.setIcon(column, c_prop)
+
                 self.traverseWidgetTree(elem)
-                self.itemstack.pop()
+                _, self.item_nr = self.itemstack.pop()
+
             elif isinstance(w, QtGui.QTableWidget):
-                item = self.factory.createQObject("QTableWidgetItem",
-                                              self.uniqueName("item"),
-                                              (), False)
-                self.wprops.setProperties(item, elem)
-                w.setItem(int(elem.attrib["row"]), int(elem.attrib["column"]), item)
-                             
-                
+                text = self.wprops.getProperty(elem, "text")
+                icon = self.wprops.getProperty(elem, "icon")
+
+                item = self.factory.createQObject("QTableWidgetItem", "item",
+                        (), False)
+
+                if self.item_nr == 0:
+                    self.sorting_enabled = self.factory.invoke("__sortingEnabled", w.isSortingEnabled)
+                    w.setSortingEnabled(False)
+
+                row = int(elem.attrib["row"])
+                col = int(elem.attrib["column"])
+
+                if text:
+                    w.item(row, col).setText(text)
+
+                if icon:
+                    item.setIcon(icon)
+
+                w.setItem(row, col, item)
+
+            self.item_nr += 1
+
     def addAction(self, elem):
         self.actions.append((self.stack.topwidget, elem.attrib["name"]))
 
     def addHeader(self, elem):
-        if isinstance(self.stack.topwidget, QtGui.QTreeWidget):
-            self.stack.topwidget.headerItem().setText(
-                self.column_counter,
-                self.wprops.getProperty(elem, "text"))
+        w = self.stack.topwidget
+
+        if isinstance(w, QtGui.QTreeWidget):
+            text = self.wprops.getProperty(elem, "text")
             icon = self.wprops.getProperty(elem, "icon")
+
+            if text:
+                w.headerItem().setText(self.column_counter, text)
+
             if icon:
-                self.stack.topwidget.headerItem().setIcon(self.column_counter, icon)
+                w.headerItem().setIcon(self.column_counter, icon)
+
             self.column_counter += 1
-        elif isinstance(self.stack.topwidget, QtGui.QTableWidget):
+
+        elif isinstance(w, QtGui.QTableWidget):
             if len(elem) == 0:
                 return
-            item = self.factory.createQObject("QTableWidgetItem",
-                                          self.uniqueName("headerItem"),
-                                          (), False)
-            self.wprops.setProperties(item, elem)
+
+            text = self.wprops.getProperty(elem, "text")
+            icon = self.wprops.getProperty(elem, "icon")
+
+            item = self.factory.createQObject("QTableWidgetItem", "item",
+                        (), False)
+
             if elem.tag == "column":
-                self.stack.topwidget.setHorizontalHeaderItem(self.column_counter, item)
+                w.setHorizontalHeaderItem(self.column_counter, item)
+
+                if text:
+                    w.horizontalHeaderItem(self.column_counter).setText(text)
+
+                if icon:
+                    item.setIcon(icon)
+
                 self.column_counter += 1
-            else:
-                self.stack.topwidget.setVerticalHeaderItem(self.row_counter, item)
+            elif elem.tag == "row":
+                w.setVerticalHeaderItem(self.row_counter, item)
+
+                if text:
+                    w.verticalHeaderItem(self.row_counter).setText(text)
+
+                if icon:
+                    item.setIcon(icon)
+
                 self.row_counter += 1
 
     def createAction(self, elem):
@@ -404,8 +597,9 @@ class UIParser(object):
         for child in iter(elem):
             try:
                 handler = self.widgetTreeItemHandlers[child.tag]
-            except KeyError, e: 
+            except KeyError: 
                 continue
+
             handler(self, child)
 
     def createUserInterface(self, elem):
@@ -482,13 +676,13 @@ class UIParser(object):
         self.defaults["spacing"] = int(elem.attrib["spacing"])
 
     def setTaborder(self, elem):
-        try:
-            lastwidget = getattr(self.toplevelWidget, elem[0].text)
-        except IndexError:
-            return
-        for widget in iter(elem[1:]):
-            widget = getattr(self.toplevelWidget, widget.text)
-            self.toplevelWidget.setTabOrder(lastwidget, widget)
+        lastwidget = None
+        for widget_elem in elem:
+            widget = getattr(self.toplevelWidget, widget_elem.text)
+
+            if lastwidget is not None:
+                self.toplevelWidget.setTabOrder(lastwidget, widget)
+
             lastwidget = widget
 
     def readResources(self, elem):
@@ -528,18 +722,29 @@ class UIParser(object):
             if header.endswith(".h"):
                 header = header[:-2]
 
-            return header.replace("/", ".")
+            mpath = []
+            for part in header.split('/'):
+                # Ignore any empty parts or those that refer to the current
+                # directory.
+                if part not in ('', '.'):
+                    if part == '..':
+                        # We should allow this for Python3.
+                        raise SyntaxError("custom widget header file name may not contain '..'.")
+
+                    mpath.append(part)
+
+            return '.'.join(mpath)
     
         for custom_widget in iter(elem):
             classname = custom_widget.findtext("class")
             if classname.startswith("Q3"):
-                raise NoSuchWidgetError, classname
+                raise NoSuchWidgetError(classname)
             self.factory.addCustomWidget(classname,
                                      custom_widget.findtext("extends") or "QWidget",
                                      header2module(custom_widget.findtext("header")))
 
     def createToplevelWidget(self, classname, widgetname):
-        raise NotImplementedError, "must be overridden"
+        raise NotImplementedError
     
     # finalize will be called after the whole tree has been parsed and can be
     # overridden.
@@ -564,7 +769,7 @@ class UIParser(object):
         version = document.getroot().attrib["version"]
         DEBUG("UI version is %s" % (version,))
         # Right now, only version 4.0 is supported, which is used up to at
-        # least Qt 4.3.
+        # least Qt 4.4.
         assert version in ("4.0",)
         for tagname, actor in branchHandlers:
             elem = document.find(tagname)
